<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2006 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: Entities.php 5631 2006-10-09 18:21:43Z andrew@m3.net $
*/

/**
 * A file that contains a number of entity classes used in the Maintenance Priority
 * process: Advert, Placement and Zone.
 */

require_once MAX_PATH . '/lib/max/core/ServiceLocator.php';
require_once MAX_PATH . '/lib/max/Dal/Maintenance/Priority.php';

/**
 * An entity class used to represent advertisements.
 *
 * @package    MaxMaintenance
 * @subpackage Priority
 * @author     Andrew Hill <andrew@m3.net>
 * @author     Demain Turner <demian@m3.net>
 * @author     James Floyd <james@m3.net>
 */
class Advert
{

    var $oDal;
    var $id;
    var $type;
    var $weight;
    var $active;
    var $requiredImpressions;
    var $requestedImpressions;
    var $zones = array();
    var $pastRequiredImpressions;
    var $pastActualImpressions;
    var $pastAdZonePriorityFactor;
    var $pastZoneTrafficFraction;

    /**
     * The constructor method.
     *
     * @param array $aAdvert An associative array of values to be assigned to
     *                       the object, array keys reflect database field names.
     */
	function Advert($aAdvert = array())
    {
        $this->oDal = $this->_getDal();
        if (count($aAdvert)) {
            $this->id     = $aAdvert['ad_id'];
            $this->type   = $aAdvert['type'];
            $this->weight = $aAdvert['weight'];
            if ($aAdvert['active'] == 't') {
                $this->active = true;
            } elseif ($aAdvert['active'] == 'f') {
                $this->active = false;
            }
        }
    }

    /**
     * A method to create, register and return the Maintenance Priority DAL.
     *
     * @access private
     * @return object MAX_Dal_Maintenance_Priority
     */
    function &_getDal()
    {
        $oServiceLocator = &ServiceLocator::instance();
        $oDal = &$oServiceLocator->get('MAX_Dal_Maintenance_Priority');
        if (!$oDal) {
            $oDal = new MAX_Dal_Maintenance_Priority();
            $oServiceLocator->register('MAX_Dal_Maintenance_Priority', $oDal);
        }
        return $oDal;
    }

    /**
     * A method to get the delivery limitations of an advertisement
     * object from the database, and return them as an array.
     *
     * @return array An array of arrays, each representing a delivery
     *               limitation, for example:
     *               array(
     *                   [ad_id]             => 1
     *                   [logical]           => and
     *                   [type]              => Time:Hour
     *                   [comparison]        => ==
     *                   [data]              => 1,7,18,23
     *                   [executionorder]    => 1
     *               )
     */
	function getDeliveryLimitations()
	{
        $wheres[] = array(  $conf['table']['prefix'] .
                            $conf['table']['acls'].'bannerid = '.$this->id, 'AND');
		return $this->oDal->getAdDeliveryLimitations(array(), $wheres);
	}

}


/**
 * An entity class used to represent placements (used to be campaigns).
 *
 * @package    MaxMaintenance
 * @subpackage Priority
 * @author     Andrew Hill <andrew@m3.net>
 * @author     Demain Turner <demian@m3.net>
 * @author     James Floyd <james@m3.net>
 */
class Placement
{

    var $id;
    var $oDal;
    var $adverts                    = array();
    var $expire;
    var $targetImpressions          = 0;
    var $targetClicks               = 0;
    var $targetConversions          = 0;
    var $dailyTargetImpressions     = 0;
    var $dailyTargetClicks          = 0;
    var $dailyTargetConversions     = 0;
    var $priority                   = 0;
    var $sumRequests                = 0;
    var $sumImpressions             = 0;
    var $sumClicks                  = 0;
    var $sumConversions             = 0;
    var $requiredImpressions        = 0;

    /**
     * The constructor method.
     *
     * @param array $aPlacement An associative array of values to be assigned to
     *                          the object, array keys reflect database field names.
     */
    function Placement($aPlacement = array())
    {
        $this->oDal                     = $this->_getDal();
        $this->id                       = (int)$aPlacement['campaignid'];
        $this->expire                   =      $aPlacement['expire'];
        $this->targetImpressions        = (int)$aPlacement['views'];
        $this->targetClicks             = (int)$aPlacement['clicks'];
        $this->targetConversions        = (int)$aPlacement['conversions'];
        $this->dailyTargetImpressions   = (int)$aPlacement['target_impression'];
        $this->dailyTargetClicks        = (int)$aPlacement['target_click'];
        $this->dailyTargetConversions   = (int)$aPlacement['target_conversion'];
        $this->priority                 = (int)$aPlacement['priority'];
    }

    /**
     * A method to create, register and return the Maintenance Priority DAL.
     *
     * @access private
     * @return object MAX_Dal_Maintenance_Priority
     */
    function &_getDal()
    {
        $oServiceLocator = &ServiceLocator::instance();
        $oDal = &$oServiceLocator->get('MAX_Dal_Maintenance_Priority');
        if (!$oDal) {
            $oDal = new MAX_Dal_Maintenance_Priority();
            $oServiceLocator->register('MAX_Dal_Maintenance_Priority', $oDal);
        }
        return $oDal;
    }

    /**
     * A method to set the placement's adverts array to contain Advert
     * object of each ad in the placement, from the data stored in the
     * database.
     *
     * @return void
     */
    function setAdverts()
    {
        $aAdverts = $this->oDal->getAdsByPlacementId($this->id);
        if (is_array($aAdverts) && (count($aAdverts) > 0)) {
            foreach ($aAdverts as $aAdvert) {
                $this->adverts[] = new Advert($aAdvert);
            }
        }
    }

    /**
     * A method to set the basic placement data and summary of delivery statistics
     * to date, for the placement object, from the data stored in the database.
     *
     * @return void
     */
    function setSummaryStatisticsToDate()
    {
        $aStats = $this->oDal->getPlacementStats($this->id, false);
        $this->sumRequests    = (int)$aStats['sum_requests'];
        $this->sumImpressions = (int)$aStats['sum_views'];
        $this->sumClicks      = (int)$aStats['sum_clicks'];
        $this->sumConversions = (int)$aStats['sum_conversions'];
    }

    /**
     * A method to set the basic placement data and summary of delivery statistics
     * for today only, for the placement object, from the data stored in the database.
     *
     * @param string $today A "CCYY-MM-DD" formatted representation of today's date.
     * @return void
     */
    function setSummaryStatisticsToday($today)
    {
        $aStats = $this->oDal->getPlacementStats($this->id, true, $today);
        $this->sumRequests    = (int)$aStats['sum_requests'];
        $this->sumImpressions = (int)$aStats['sum_views'];
        $this->sumClicks      = (int)$aStats['sum_clicks'];
        $this->sumConversions = (int)$aStats['sum_conversions'];
    }

}


/**
 * An entity class used to represent zones.
 *
 * @package    MaxMaintenance
 * @subpackage Priority
 * @author     Andrew Hill <andrew@m3.net>
 * @author     Demain Turner <demian@m3.net>
 * @author     James Floyd <james@m3.net>
 */
class Zone
{

    var $id;
    var $availableImpressions = 0;
    var $averageImpressions   = 0;
    var $aAdverts             = array();
    var $aOperationIntId      = array();
    var $pastActualImpressions;

    /**
     * The constructor method.
     *
     * @param array $aZone associative array of values to be assigned to
     *              object, array keys reflect database field names
     */
    function Zone($aZone = array())
    {
        $this->id = (int)$aZone['zoneid'];
    }

    /**
     * A method to add Advert objects to the Zone.
     *
     * @param Advert $oAdvert The Advert object to add.
     * @return void
     */
    function addAdvert($oAdvert)
    {
        $this->aAdverts[$oAdvert->id] = $oAdvert;
    }

    /**
     * Method to set the average impressions for a given
     * operation interval ID
     *
     * @param integer $intervalId
     * @param integer $impressions
     * @return void
     */
    function setIntervalIdImpressionAverage($intervalId, $impressions = 0)
    {
        if (isset($intervalId)) {
            $this->aOperationIntId[$intervalId]['averageImpressions'] = $impressions;
        }
    }

    /**
     * Method to set the forecast impressions for a given
     * operation interval ID
     *
     * @param integer $intervalId
     * @param integer $impressions
     * @return void
     */
    function setIntervalIdImpressionForecast($intervalId, $impressions = 0)
    {
        if (isset($intervalId)) {
            $this->aOperationIntId[$intervalId]['forecastImpressions'] = $impressions;
        }
    }

    /**
     * Method to set the actual impressions value for a given
     * operation interval ID
     *
     * @param integer $intervalId
     * @param integer $impressions
     * @return void
     */
    function setIntervalIdImpressionActual($intervalId, $impressions = 0)
    {
        if (isset($intervalId)) {
            $this->aOperationIntId[$intervalId]['actualImpressions'] = $impressions;
        }
    }

}

?>
